
/**
 * 校验是否是合法的手机号
 * @param input 输入
 */
function validPhone(input: string | number): boolean {
  return /^\d+$/.test(`${input || ''}`);
}

/**
 * 校验手机号的输入是否合法（重点是输入过程中）
 * @param input 输入
 */
function validPhoneInput(input: string | number): boolean {
  return /^\d*$/.test(`${input || ''}`);
}

/**
 * 校验是否是合法的密码
 * @param input 输入
 */
function validPwd(input: string | number): boolean {
  return /^[0-9a-zA-Z]{6,18}$/.test(`${input || ''}`);
}

/**
 * 校验密码的输入是否合法（重点是输入过程中）
 * @param input 输入
 */
function validPwdInput(input: string | number): boolean {
  return /^[0-9a-zA-Z]*$/.test(`${input || ''}`);
}

export {
  validPhone,
  validPhoneInput,
  validPwd,
  validPwdInput,
}
